function [err_sq_el,xx,fe,ade] = stoch_diffpost_q2_ypx(xy,mv,mbound,q2sol,...
    eex,hx,hy,indset,P,norv,noarv,KL_DATA,tout)
%STOCH_DIFFPOST_Q2_YPX computes YPX error estimator for stochastic Q2 solution
% V_{YP}-estimator: employs elementwise Q2-bubbles for the edge midpoints and centroid
%                   tensorised with the original set of polynomials,
%                   uses 'mean-based' bilinear form B_0
%   [zerr_sq_el,zelerr,zfe,zae] = stoch_diffpost_q1_ypx(xy,ev,ebound,x_gal,...
%                                  eex,hx,hy,indset,P,norv,noarv,KL_DATA,tout);
%   input
%          xy           vertex coordinate vector
%          ev           element mapping matrix
%          ebound       element edge boundary matrix
%          x_gal        Q1 solution vector
%          eex          element edge connectivity array
%          hx,hy        element mesh sizes
%          indset       index set of polynomial degrees
%          P            length of the index set
%          norv         number of random variables
%          noarv        the number of active random variables
%          KL_DATA      data related to KL-expansion
%          tout         output switch
%   output
%          zerr_sq_el   element error estimate
%          zelerr       elementwise error estimate
%          zfe          elementwise rhs vectors
%          zae          LDLT factorized element matrices
%
%   calls functions q1fluxjmps, gausspoints_oned, gausspoints_twod
%   SIFISS function:  DJS 5 July 2017
% Copyright (c) 2017 A. Crowder, D. Silvester


fprintf('computing YP-error estimator...  \n')
x=xy(:,1); y=xy(:,2);
nel=length(mv(:,1));

err_sq_el = zeros(nel,1);
xl_v=nan(nel,4); yl_v=nan(nel,4);

% 12 error nodes - excluding nodes 19,21,23,25 from original code
error_node=[10;11;12;13;14;15;16;17;18;20;22;24];

N_error_node=length(error_node);
nn = N_error_node;


% construct the integration rule
ngpt=3; 
[oneg,onew] = gausspoints_oned(ngpt);
[s,t,wt] = gausspoints_twod(oneg,onew);
nngpt=ngpt^2; ng=ngpt;
t1=tic;
%
% inner loop over elements
for ivtx = 1:4
    xl_v(:,ivtx) = x(mv(:,ivtx));
    yl_v(:,ivtx) = y(mv(:,ivtx));
end

% initialise local stochastic and deterministic matrices
ade = zeros(nel,nn,nn);
elerr=zeros(nn,nel);
adem = zeros(nel,4,5,5);

xl_s = zeros(nel,4,4);
yl_s = zeros(nel,4,4);

% compute local mid-edge coordinates
xedge1(:,1)= 0.5*(xl_v(:,1)+xl_v(:,2)); yedge1(:,1)= 0.5*(yl_v(:,1)+yl_v(:,2));
xedge2(:,1)= 0.5*(xl_v(:,2)+xl_v(:,3)); yedge2(:,1)= 0.5*(yl_v(:,2)+yl_v(:,3));
xedge3(:,1)= 0.5*(xl_v(:,3)+xl_v(:,4)); yedge3(:,1)= 0.5*(yl_v(:,3)+yl_v(:,4));
xedge4(:,1)= 0.5*(xl_v(:,4)+xl_v(:,1)); yedge4(:,1)= 0.5*(yl_v(:,4)+yl_v(:,1));
xcentr(:,1)= 0.25*(xedge1(:)+xedge2(:)+xedge3(:)+xedge4(:));
ycentr(:,1)= 0.25*(yedge1(:)+yedge2(:)+yedge3(:)+yedge4(:));
if tout>0,
    elt=27;
    fprintf('\n\n test element check \n')
    fprintf('vertex x-coordinates \n')
    disp(xl_v(elt,:))
    fprintf('vertex y-coordinates \n')
    disp(yl_v(elt,:))
end

% Save coordinates of macroelements numbered [4 3; 1 2]

% first
xl_s(:,1,1) = xl_v(:,1); yl_s(:,1,1) = yl_v(:,1);
xl_s(:,1,2) = xedge1(:); yl_s(:,1,2) = yedge1(:);
xl_s(:,1,3) = xcentr(:); yl_s(:,1,3) = ycentr(:);
xl_s(:,1,4) = xedge4(:); yl_s(:,1,4) = yedge4(:);
% second
xl_s(:,2,1) = xedge1(:); yl_s(:,2,1) = yedge1(:);
xl_s(:,2,2) = xl_v(:,2); yl_s(:,2,2) = yl_v(:,2);
xl_s(:,2,3) = xedge2(:); yl_s(:,2,3) = yedge2(:);
xl_s(:,2,4) = xcentr(:); yl_s(:,2,4) = ycentr(:);
% third
xl_s(:,3,1) = xcentr(:); yl_s(:,3,1) = ycentr(:);
xl_s(:,3,2) = xedge2(:); yl_s(:,3,2) = yedge2(:);
xl_s(:,3,3) = xl_v(:,3); yl_s(:,3,3) = yl_v(:,3);
xl_s(:,3,4) = xedge3(:); yl_s(:,3,4) = yedge3(:);
% fourth
xl_s(:,4,1) = xedge4(:); yl_s(:,4,1) = yedge4(:);
xl_s(:,4,2) = xcentr(:); yl_s(:,4,2) = ycentr(:);
xl_s(:,4,3) = xedge3(:); yl_s(:,4,3) = yedge3(:);
xl_s(:,4,4) = xl_v(:,4); yl_s(:,4,4) = yl_v(:,4);

% Construct 5x5 stiffness matrices on each macroelement using Q2 functions
% 5 to 9
% inner-inner loop over subdivided elements
for subelt=1:4
    for ivtx = 1:4
        xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
        yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
    end
    % loop over Gauss points
    for igpt = 1:nngpt
        sigpt=s(igpt);
        tigpt=t(igpt);
        wght=wt(igpt);
        % evaluate derivatives etc
        [~,invjac_v,~,~,~] = deriv(sigpt,tigpt,xl_m,yl_m);
        [~,dphidx_v,dphidy_v] = qderiv(sigpt,tigpt,xl_m,yl_m);
        
        coeff_v = stoch_gauss_coeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
        for j = 1:5
            for i = 1:5
                adem(:,subelt,i,j) = adem(:,subelt,i,j)+wght*coeff_v(:,1).*dphidx_v(:,i+4).*dphidx_v(:,j+4).*invjac_v(:);
                adem(:,subelt,i,j) = adem(:,subelt,i,j)+wght*coeff_v(:,1).*dphidy_v(:,i+4).*dphidy_v(:,j+4).*invjac_v(:);
            end
        end
        % end of Gauss point loop
    end
    % debug
    if tout >0,
        fprintf('\n\n test Q2 stiffness matrix \n')
        fprintf('subelement : %g \n',subelt)
        rats(squeeze(adem(elt,subelt,1:5,1:5)))
    end
    % end of subdivided element loop
end

% Construct the `global' 12x12 stiffness matrix on each element
% Error node 1 nonzero interactions
ade(:,1,1) = adem(:,1,1,1);
ade(:,1,8) = adem(:,1,1,4);
ade(:,1,9) = adem(:,1,1,5);

% Error node 2 nonzero interactions
ade(:,2,2) = adem(:,2,1,1);
ade(:,2,3) = adem(:,2,1,2);
ade(:,2,10) = adem(:,2,1,5);

% 3
ade(:,3,3) = adem(:,2,2,2);
ade(:,3,2) = adem(:,2,2,1);
ade(:,3,10) = adem(:,2,2,5);

% 4
ade(:,4,4) = adem(:,3,2,2);
ade(:,4,5) = adem(:,3,2,3);
ade(:,4,11) = adem(:,3,2,5);

% 5
ade(:,5,5) = adem(:,3,3,3);
ade(:,5,4) = adem(:,3,3,2);
ade(:,5,11) = adem(:,3,3,5);

% 6
ade(:,6,6) = adem(:,4,3,3);
ade(:,6,7) = adem(:,4,3,4);
ade(:,6,12) = adem(:,4,3,5);

% 7
ade(:,7,7) = adem(:,4,4,4);
ade(:,7,6) = adem(:,4,4,3);
ade(:,7,12) = adem(:,4,4,5);

% 8
ade(:,8,8) = adem(:,1,4,4);
ade(:,8,1) = adem(:,1,4,1);
ade(:,8,9) = adem(:,1,4,5);

% 9
ade(:,9,9) = adem(:,1,5,5);
ade(:,9,1) = adem(:,1,5,1);
ade(:,9,8) = adem(:,1,5,4);

% 10
ade(:,10,10) = adem(:,2,5,5);
ade(:,10,2) = adem(:,2,5,1);
ade(:,10,3) = adem(:,2,5,2);

% 11
ade(:,11,11) = adem(:,3,5,5);
ade(:,11,4) = adem(:,3,5,2);
ade(:,11,5) = adem(:,3,5,3);

% 12
ade(:,12,12) = adem(:,4,5,5);
ade(:,12,6) = adem(:,4,5,3);
ade(:,12,7) = adem(:,4,5,4);

if tout>0,
    fprintf('\n\n test assembled 16x16 stiffness matrix \n')
    fprintf('element : %g \n',elt)
    rats(squeeze(ade(elt,1:16,1:16)))
end

% Compute right-hand side residuals
res_int = stoch_intres_q2_yp(xy,xl_s,yl_s,mv,q2sol,indset,P,norv,noarv,KL_DATA);
res_edge = stoch_edgeres_q2_ypx(xy,xl_s,yl_s,mv,mbound,q2sol,eex,hx,hy,indset,P,norv,noarv,KL_DATA);

fprintf('yp estimate | res_int = %7.4e : res_edge = %7.4e\n',norm(res_int), norm(res_edge))
fe = res_int - res_edge;

% debug
if tout>0,
    fprintf('right-hand side vector \n')
    disp(squeeze(fe(elt,1:end)))
end

% solve for local estimate (vectorized code)
% LDLT factorization

dd=zeros(nel,nn); rr=zeros(nel,nn);
for kk=1:nn-1
    for pp=1:kk-1;
        rr(1:nel,pp)=dd(1:nel,pp).*ade(1:nel,kk,pp);
    end
    dd(1:nel,kk)=ade(1:nel,kk,kk);
    for pp=1:kk-1;
        dd(1:nel,kk)= dd(1:nel,kk)- ade(1:nel,kk,pp).*rr(1:nel,pp);
    end
    for ii=kk+1:nn
        for pp=1:kk-1;
            ade(1:nel,ii,kk)=ade(1:nel,ii,kk)-ade(1:nel,ii,pp).*rr(1:nel,pp);
        end
        ade(1:nel,ii,kk)=ade(1:nel,ii,kk)./dd(1:nel,kk);
    end
end
for pp=1:nn-1;
    rr(1:nel,pp)=dd(1:nel,pp).*ade(1:nel,nn,pp);
end
dd(1:nel,nn)=ade(1:nel,nn,nn);
for pp=1:nn-1;
    dd(1:nel,nn)= dd(1:nel,nn)- ade(1:nel,nn,pp).*rr(1:nel,pp);
end
% overwrite diagonal entries
for kk=1:nn
    ade(1:nel,kk,kk)= dd(1:nel,kk);
end
% forward-backward substitutions ...
% loop over the stochastic components
elerr=zeros(nn*P,nel);
for nnp=1:P;
    fde=fe(:,(nnp-1)*nn+1:nnp*nn);
    if tout>0, fprintf('%g right-hand side\n',nnp),
        disp(squeeze(fde(elt,1:end))),end
    xdx = element_lusolve(ade,fde);
    elerr((nnp-1)*nn+1:nnp*nn,1:nel)=xdx';
end
xx=elerr';

for svtx=1:nn*P
    err_sq_el(:) = err_sq_el(:) + fe(:,svtx) .* elerr(svtx,:)';
end

etime=toc(t1);
fprintf('  Error estimation took %6.3e seconds\n',etime)



return

