function [err_sq_el,elerr,fe,ae] = stoch_diffpost_q2_yqx(xy,mv,mbound,q2sol,...
                                   eex,hx,hy,indset,polyd,norv,noarv,KL_DATA,pm_choice,tout)
%STOCH_DIFFPOST_Q2_YQX  computes YQX error estimator for stochastic Q2 solution
% V_{YQ}-estimator: employs elementwise Q4-bubbles
%                   tensorised with the 'detailed set' of polynomials,
%                   uses 'mean-based' bilinear form B_0
%   [err_sq_el,elerr,fe,ae] = stoch_diffpost_q2_yqx(xy,mv,mbound,x_gal,...
%                             eex,hx,hy,indset,polyd,norv,noarv,KL_DATA,pm_choice,tout);
%   input
%          xy             vertex coordinate vector
%          mv             element mapping matrix
%          mbound         element edge boundary matrix
%          x_gal          stochastic Q1 solution vector
%          eex            element edge connectivity array
%          hx,hy          element mesh sizes
%          indset         index set of polynomial degrees
%          polyd          maximum polynomial degree in the index set
%          norv           number of random variables
%          noarv          the number of active random variables
%          KL_DATA        data related to KL-expansion
%          pm_choice      p- or M- method for enriching the polynomial space
%          tout           output switch
%   output
%          err_sq_el      element error estimate (squared 'energy error')
%          elerr          elementwise error estimate coefficient vector
%          fe             elementwise rhs vectors
%          ae             LDLT factorized element matrices
%
%   calls functions gausspoints_oned, gausspoints_twod, deriv, qderiv
%   calls functions stoch_intres_q2_yq, stoch_edgeres_q2_yqx, stoch_gmatrices
%   SIFISS function: DJS; 5 July 2017
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

% check the index set
 [P,norvx] = size(indset); 
 if norvx~=norv, error('error in call to stoch_diffpost_q2_yqx!'), end

%% taken from diffpost_q2_with_q4
reduction_level=3;
error_node=[10;11;12;13;14;15;16;17;18;20;22;24];
N_error_node=length(error_node); nn = N_error_node;

  fprintf('computing YQ-error estimator...  \n')

% set new index set depending on pm_choice
  if pm_choice==1
% set new index set by increasing the polynomial degree
     [new_indset] = stoch_indset(polyd+1,noarv);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv)];
     if tout >0,fprintf('detail space polynomial degree is %g\n',polyd+1),end
  elseif pm_choice==2
% set new index set by increasing the number of active random variables
     [new_indset] = stoch_indset(polyd,noarv+1);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv-1)];
     if tout >0,fprintf('detail space is based on %g random variables\n',noarv+1),end
  elseif pm_choice==3
% set new index set by increasing both the polynomial degree and the number of active random variables
     [new_indset] = stoch_indset(polyd+1,noarv+1);
     new_indset = [new_indset,zeros(length(new_indset),norv-noarv-1)];
     if tout >0,fprintf('detail space is based on %g random variables, polynomial degree is %i\n',noarv+1,polyd+1),end
  end


% find the 'difference' between the new and original index sets
  Q_indset = setdiff(new_indset,indset,'rows');

% find the length of Q_indset
  [Q,xnorv] = size(Q_indset);

  x=xy(:,1); y=xy(:,2);
  mel=length(mv(:,1));
  err_sq_el = zeros(mel,1);
  xl_v=nan(mel,4); yl_v=nan(mel,4);


% construct the integration rule
ngpt=7;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2; ng=ngpt;
  t1=tic;
% inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(mv(:,ivtx));
      yl_v(:,ivtx) = y(mv(:,ivtx)); 
  end

% debug
if tout>0,
elt=27;
fprintf('\n\n test element check \n')
fprintf(' element : %g \n',elt)
fprintf('vertex x-coordinates \n')
disp(xl_v(elt,:))
fprintf('vertex y-coordinates \n')
disp(yl_v(elt,:))
end

% initialise local stochastic and deterministic matrices
  ae = zeros(mel,nn,nn);
  elerr=zeros(nn,mel);

% loop over Gauss points
   for igpt = 1:nngpt
      sigpt=s(igpt);
      tigpt=t(igpt);
      wght=wt(igpt);
% evaluate derivatives etc
      [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigpt,tigpt,xl_v,yl_v);
      [qua_v,dquadx_v,dquady_v] = qqderiv(sigpt,tigpt,xl_v,yl_v);
      for j = 1:N_error_node
         jj=error_node(j);
         for i = 1:N_error_node
            ii=error_node(i);
            ae(:,i,j) = ae(:,i,j)+wght*dquadx_v(:,ii).*dquadx_v(:,jj).*invjac_v(:);
            ae(:,i,j) = ae(:,i,j)+wght*dquady_v(:,ii).*dquady_v(:,jj).*invjac_v(:);
         end
      end
% end of Gauss point loop
   end


% compute G-matrices
  G = stoch_gmatrices(Q_indset,Q,noarv,norv);

% assemble local stiffness matrices by tensorising with G-matrices
%  [nzi,nzj,g] = find(G{1});
%  for t=1:length(nzi)
%      i=nzi(t); j=nzj(t);
%      ae(:,(nn*(i-1)+1):(nn*i),(nn*(j-1)+1):(nn*j)) = ae(:,(nn*(i-1)+1):(nn*i),(nn*(j-1)+1):(nn*j)) + g(t)*ae(:,1:nn,1:nn);
%  end

% compute local rhs-vectors
res_int = stoch_intres_q2_yq(xy,mv,q2sol,indset,P,Q_indset,norv,noarv,KL_DATA);
res_edge = stoch_edgeres_q2_yqx(xy,mv,mbound,q2sol,eex,hx,hy,indset,P,Q_indset,noarv,KL_DATA);
fprintf('yq estimate | res_int = %7.4e : res_edge = %7.4e\n',norm(res_int), norm(res_edge))
fe = res_int - res_edge;
 

% solve for local estimate (sequential code)
%  for ielem = 1:mel
%      elerr(:,ielem) = squeeze(ae(ielem,1:nn,1:nn))\(fe(ielem,1:nn)');
%  end
%
% solve for local estimate (vectorised code)
% LDLT factorization
  dd=zeros(mel,nn); rr=zeros(mel,nn);
  for kk=1:nn-1
      for pp=1:kk-1;
          rr(1:mel,pp)=dd(1:mel,pp).*ae(1:mel,kk,pp);
      end
      dd(1:mel,kk)=ae(1:mel,kk,kk);
      for pp=1:kk-1;
          dd(1:mel,kk)= dd(1:mel,kk)- ae(1:mel,kk,pp).*rr(1:mel,pp);
      end
      for ii=kk+1:nn
          for pp=1:kk-1;
              ae(1:mel,ii,kk)=ae(1:mel,ii,kk)-ae(1:mel,ii,pp).*rr(1:mel,pp);
          end
          ae(1:mel,ii,kk)=ae(1:mel,ii,kk)./dd(1:mel,kk);
      end
  end
  for pp=1:nn-1;
      rr(1:mel,pp)=dd(1:mel,pp).*ae(1:mel,nn,pp);
  end
  dd(1:mel,nn)=ae(1:mel,nn,nn);
  for pp=1:nn-1;
      dd(1:mel,nn)= dd(1:mel,nn)- ae(1:mel,nn,pp).*rr(1:mel,pp);
  end
% overwrite diagonal entries
  for kk=1:nn
      ae(1:mel,kk,kk)= dd(1:mel,kk);
  end
% forward-backward substitutions ...
% loop over the stochastic components
  elerr=zeros(nn*Q,mel); 
  for nnp=1:Q;
  fde=fe(:,(nnp-1)*nn+1:nnp*nn);
    if tout>1 & nnp==Q, fprintf('%g right-hand side\n',nnp),
    disp(squeeze(fde(elt,1:end))),end
  xdx = element_lusolve(ae,fde);
  elerr((nnp-1)*nn+1:nnp*nn,1:mel)=xdx';
  end
  xx=elerr';
%
  for svtx=1:nn*Q
      err_sq_el(:) = err_sq_el(:) + fe(:,svtx) .* elerr(svtx,:)';
  end

  etime=toc(t1);
  fprintf('  Error estimation took %6.3e seconds\n',etime)

  return

