function [x,resvec,emin,errpde,info] = ...
                   stoch_est_minres(G,K,fgal,apost,stopit,maxit,tol,...
                              prob_type,MA,aparams,Mest,eparams);
% MINRES for stochastic diffusion problems with discretization error estimation
% [x,resvec,emin,errpde,info]= ...
%        stoch_est_minres(G,K,fgal,apost,stopit,maxit,tol,...
%                   prob_type,MA,aparams,Mest,eparams);
%   input
%         G,K             cell structure with nrv component matrices
%         fgal            RHS vector 
%         apost           1/0 error estimation on/off  
%         stopit          1/0 stopping test on/off
%         maxit           max number of iterations
%         tol             stopping tolerance
%         prob_type       problem  type switch
%         MA,aparams      preconditioner (function_name, structure)
%         Mest,eparams    error estimator 
%   output
%         x            iterative solution
%         resvec       vector of redidual errors
%         emin         vector of smallest eigenvalue estimates
%         errpde       vector of  discretization error estimates
%         info         computed eigenvalue estimates
% calls function stoch_est
%   SIFISS function: DJS; 1 April 2015.
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester

checkcode=1;
apost=0; stopit=1; %  aposteriori error estimation is not enabled
fprintf('Call to STOCH_EST_MINRES without error control ...\n')
Ahandle=str2func(MA);  
  
rhs=fgal;x0=zeros(size(rhs));
na=length(fgal); n=na;
%
emin=[nan,nan,nan]; emax=[nan,nan,nan];
b=rhs;
x = x0;
%res0=b-Agal*x; 
% matrix-vector product
res0 = b- stoch_matvec(x,G,K);
res=zeros(n,1); 
res(1:na)=Ahandle(res0(1:na),aparams);    
arr=sqrt( res0'*res); 
erro=arr;
resvec(1)=arr; errpde(1)=nan;

   R = zeros(4,1);

% Q contains rotations. c_k, s_k could be stored instead
   cc=zeros(2,1);ss=zeros(2,1);

% Lanczos vectors
   w0=zeros(n,1);

% Auxiliary vectors for updating solution
   p0=zeros(n,1); p=zeros(n,1);
   v = res/arr; 
   w = res0/arr;
   F=arr*speye(2,1);
   i=0; tstart=cputime;
   
   while (i<maxit & erro>tol) %----------- start of iteration loop
     i=i+1;
     i1=i+1;
% Matrix-vector product
%    w1 = Agal*v;
     w1 = stoch_matvec(v,G,K);
     p1=v;
     if (i>1)
         H(i-1,i)=H(i,i-1);
         w1 = w1 - w0*H(i-1,i);
     end
     H(i,i)=v'*w1;
     w1 = w1 - w*H(i,i);

% Apply left precond P using P inner product for V.
     v(1:na,1)=Ahandle(w1(1:na),aparams);
     H(i+1,i)=sqrt(v'*w1);
     if (H(i+1,i) ~= 0.)
        v=v/H(i+1,i);     w1=w1/H(i+1,i);
     else,return,end;
%
    if i>2;
%   compute the ritz values
    [coef,lambda_min,lambda_max] = stoch_est_ritz(H,i,prob_type);
    emin=[emin,lambda_min]; emax=[emax,lambda_max];
    end

% Apply previous rotations
    if (i==1), R(3)=H(1,1); end
    if (i==2),
     R(2:3,1)=GG*[H(1:i,i)];
     GGold=GG(:,2);
    end
    if (i>2),
     R(1,1)=GGold(1)*H(i-1,i); R(2,1)=GGold(2)*H(i-1,i);
     R(2:3,1)=GG*[R(2);H(i,i)];
     GGold=GG(:,2);
    end

% Determine and apply new rotations
    nrm=sqrt(R(3)^2+H(i1,i)^2);
    GG(1,1) = abs(R(3))/nrm; GG(1,2)=R(3)/abs(R(3))*conj(H(i1,i))/nrm;
    GG(2,1) = -conj(GG(1,2)); GG(2,2)=GG(1,1);
    R(3)   = GG(1,:) *[R(3);H(i1,i)];
    F(1:2)     = GG *F(1:2);

% Update approx solution
    p1 = (p1 - p0*R(1)-p*R(2))/R(3);
    x  = x + p1*F(1);
    F(1:2)=[F(2);0];
    w0=w; w=w1;
    p0=p; p=p1;
    erro=full(abs(F(1)));
    resvec=[resvec,erro];
    
   end   %------------- end of iteration loop

etoc=cputime-tstart;
fprintf('Bingo!\n')
fprintf('\nconvergence in %3i iterations\n',i)
fprintf('  %9.4e seconds\n\n',etoc)
info=[emin(i),emax(i)];  %% returns eigenvalue estimates via info
if checkcode,
   fprintf('\n Eigenvalue convergence')
   fprintf('\n    k     lambda     Lambda \n')
      for its=4:i+1,
      fprintf('%5i %11.4f %11.4f \n', its-1, emin(its-1), emax(its-1));
      end  
   fprintf('\n')
end
errpde=[];
info=[info,nan,i+1];
return