function [intres] = stoch_intres_q1_yq(xy,xl_s,yl_s,ev,q1sol,indset1,P1,indset2,norv,noarv,KL_DATA)
%STOCH_INTRES_Q1_YQ  computes YQ elementwise interior residuals for stochastic Q1 solution
%   [intres] = stoch_intres_q1_yq(xy,ev,q1sol,indset,P,norv,noarv,KL_DATA);
%   input
%          xy         vertex coordinate vector
%          xl_s
%          yl_s
%          ev         element mapping matrix
%          q1sol      stochastic Q1 solution vector
%          indset1    first index set (for the stochastic solution)
%          P1         length of the first index set
%          indset2    second index set (for the error estimator)
%          norv       number of random variables
%          noarv      the number of active random variables
%          KL_DATA    data related to KL-expansion
%   output
%          intres     interior residuals
%
%   calls functions gausspoints_oned, gausspoints_twod, deriv, qderiv
%   calls functions stoch_gauss_source, stoch_gauss_gradcoeff, stoch_rhs_multiplier
%   SIFISS function: DJS 5 July 2017 (modifications of T. Gergelits) (modification of L. Rocchi, 05 Sept 2017)
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester


  x=xy(:,1); y=xy(:,2); nvtx=length(x);
  nel=length(ev(:,1));
  [P2,norv] = size(indset2);

% construct the integration rule
  ngpt=3;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2; ng=ngpt;

% inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(ev(:,ivtx));
      yl_v(:,ivtx) = y(ev(:,ivtx));
      for k=1:P1
          sl_v(:,ivtx+4*(k-1)) = q1sol(nvtx*(k-1)+ev(:,ivtx));
      end
  end

% initialisation
  intres = zeros(nel,5*P2);
  fde = zeros(nel,5,noarv+1);
  bde = zeros(nel,5,4,noarv+1);
%
  bdem = zeros(nel,4,4,4,noarv+1);
  fdem = zeros(nel,4,4,noarv+1);


% explicit 'loop' over subdivided elements
% 1st sub-element: [-1,0]x[-1,0]
  subelt=1;
      for ivtx = 1:4
          xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
          yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
      end
% loop over Gauss points
      for igpt = 1:nngpt
          sigpt=s(igpt);
          tigpt=t(igpt);
          sigptloc=(sigpt-1.0e0)/2;
          tigptloc=(tigpt-1.0e0)/2;
          wght=wt(igpt);
% evaluate derivatives etc
          [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigptloc,tigptloc,xl_v,yl_v);
          [jac_m,invjac_m,phi_m,dphidx_m,dphidy_m] = deriv(sigpt,tigpt,xl_m,yl_m);
          rhs_m = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
          [dcoeffdx_m,dcoeffdy_m] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
          for m = 0:noarv
              for j = 1:4  % sub-element vertices
% compute rhs-contributions
                  fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs_m(:,m+1).*phi_m(:,j).*jac_m(:);
% compute div(a*grad)-contributions
                  for i = 1:4  % element vertices
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdx_m(:,m+1).*dphidx_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdy_m(:,m+1).*dphidy_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                  end
              end
          end
% end of Gauss point loop
      end

% 2nd sub-element: [0,1]x[-1,0]
  subelt=2;
      for ivtx = 1:4
          xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
          yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
      end
% loop over Gauss points
      for igpt = 1:nngpt
          sigpt=s(igpt);
          tigpt=t(igpt);
          sigptloc=(sigpt+1.0e0)/2;
          tigptloc=(tigpt-1.0e0)/2;
          wght=wt(igpt);
% evaluate derivatives etc
          [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigptloc,tigptloc,xl_v,yl_v);
          [jac_m,invjac_m,phi_m,dphidx_m,dphidy_m] = deriv(sigpt,tigpt,xl_m,yl_m);
          rhs_m = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
          [dcoeffdx_m,dcoeffdy_m] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
          for m = 0:noarv
              for j = 1:4   % sub-element vertices
% compute rhs-contributions
                  fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs_m(:,m+1).*phi_m(:,j).*jac_m(:);
% compute div(a*grad)-contributions
                  for i = 1:4   % element vertices
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdx_m(:,m+1).*dphidx_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdy_m(:,m+1).*dphidy_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                  end
              end
          end
% end of Gauss point loop
      end

% 3rd sub-element: [0,1]x[0,1]
  subelt=3;
      for ivtx = 1:4
          xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
          yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
      end
% loop over Gauss points
      for igpt = 1:nngpt
          sigpt=s(igpt);
          tigpt=t(igpt);
          sigptloc=(sigpt+1.0e0)/2;
          tigptloc=(tigpt+1.0e0)/2;
          wght=wt(igpt);
% evaluate derivatives etc
          [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigptloc,tigptloc,xl_v,yl_v);
          [jac_m,invjac_m,phi_m,dphidx_m,dphidy_m] = deriv(sigpt,tigpt,xl_m,yl_m);
          rhs_m = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
          [dcoeffdx_m,dcoeffdy_m] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
          for m = 0:noarv
              for j = 1:4   % sub-element vertices
% compute rhs-contributions
                  fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs_m(:,m+1).*phi_m(:,j).*jac_m(:);
% compute div(a*grad)-contributions
                  for i = 1:4   % element vertices
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdx_m(:,m+1).*dphidx_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdy_m(:,m+1).*dphidy_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                  end
              end
          end
% end of Gauss point loop
      end

% 4th sub-element: [-1,0]x[0,1]
  subelt=4;
      for ivtx = 1:4
          xl_m(:,ivtx) = xl_s(:,subelt,ivtx);
          yl_m(:,ivtx) = yl_s(:,subelt,ivtx);
      end
% loop over Gauss points
      for igpt = 1:nngpt
          sigpt=s(igpt);
          tigpt=t(igpt);
          sigptloc=(sigpt-1.0e0)/2;
          tigptloc=(tigpt+1.0e0)/2;
          wght=wt(igpt);
% evaluate derivatives etc
          [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigptloc,tigptloc,xl_v,yl_v);
          [jac_m,invjac_m,phi_m,dphidx_m,dphidy_m] = deriv(sigpt,tigpt,xl_m,yl_m);
          rhs_m = stoch_gauss_source(sigpt,tigpt,xl_m,yl_m,noarv);
          [dcoeffdx_m,dcoeffdy_m] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_m,yl_m,noarv,KL_DATA);
          for m = 0:noarv
              for j = 1:4   % sub-element vertices
% compute rhs-contributions
                  fdem(:,subelt,j,m+1) = fdem(:,subelt,j,m+1) + wght*rhs_m(:,m+1).*phi_m(:,j).*jac_m(:);
% compute div(a*grad)-contributions
                  for i = 1:4   % element vertices
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdx_m(:,m+1).*dphidx_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                      bdem(:,subelt,j,i,m+1) = bdem(:,subelt,j,i,m+1)+wght*dcoeffdy_m(:,m+1).*dphidy_v(:,i).*phi_m(:,j)     .*invjac_v(:).*jac_m(:); % LR modification
                  end
              end
          end
% end of Gauss point loop
      end


% manual assembly of subelement contributions
% loop over random variables
  for m = 0:noarv
% first edge
        bde(:,1,1,m+1) = bdem(:,1,2,1,m+1) + bdem(:,2,1,1,m+1);
        bde(:,1,2,m+1) = bdem(:,1,2,2,m+1) + bdem(:,2,1,2,m+1);
        bde(:,1,3,m+1) = bdem(:,1,2,3,m+1) + bdem(:,2,1,3,m+1);
        bde(:,1,4,m+1) = bdem(:,1,2,4,m+1) + bdem(:,2,1,4,m+1);
        fde(:,1,m+1) = fdem(:,1,2,m+1) + fdem(:,2,1,m+1);
% second edge
        bde(:,2,1,m+1) = bdem(:,2,3,1,m+1) + bdem(:,3,2,1,m+1);
        bde(:,2,2,m+1) = bdem(:,2,3,2,m+1) + bdem(:,3,2,2,m+1);
        bde(:,2,3,m+1) = bdem(:,2,3,3,m+1) + bdem(:,3,2,3,m+1);
        bde(:,2,4,m+1) = bdem(:,2,3,4,m+1) + bdem(:,3,2,4,m+1);
        fde(:,2,m+1) = fdem(:,2,3,m+1) + fdem(:,3,2,m+1);
% third edge
        bde(:,3,1,m+1) = bdem(:,3,4,1,m+1) + bdem(:,4,3,1,m+1);
        bde(:,3,2,m+1) = bdem(:,3,4,2,m+1) + bdem(:,4,3,2,m+1);
        bde(:,3,3,m+1) = bdem(:,3,4,3,m+1) + bdem(:,4,3,3,m+1);
        bde(:,3,4,m+1) = bdem(:,3,4,4,m+1) + bdem(:,4,3,4,m+1);
        fde(:,3,m+1) = fdem(:,3,4,m+1) + fdem(:,4,3,m+1);
% fourth edge
        bde(:,4,1,m+1) = bdem(:,4,1,1,m+1) + bdem(:,1,4,1,m+1);
        bde(:,4,2,m+1) = bdem(:,4,1,2,m+1) + bdem(:,1,4,2,m+1);
        bde(:,4,3,m+1) = bdem(:,4,1,3,m+1) + bdem(:,1,4,3,m+1);
        bde(:,4,4,m+1) = bdem(:,4,1,4,m+1) + bdem(:,1,4,4,m+1);
        fde(:,4,m+1) = fdem(:,4,1,m+1) + fdem(:,1,4,m+1);
% centroid
        bde(:,5,1,m+1) = bdem(:,1,3,1,m+1) + bdem(:,2,4,1,m+1)+ bdem(:,3,1,1,m+1) + bdem(:,4,2,1,m+1);
        bde(:,5,2,m+1) = bdem(:,1,3,2,m+1) + bdem(:,2,4,2,m+1)+ bdem(:,3,1,2,m+1) + bdem(:,4,2,2,m+1);
        bde(:,5,3,m+1) = bdem(:,1,3,3,m+1) + bdem(:,2,4,3,m+1)+ bdem(:,3,1,3,m+1) + bdem(:,4,2,3,m+1);
        bde(:,5,4,m+1) = bdem(:,1,3,4,m+1) + bdem(:,2,4,4,m+1)+ bdem(:,3,1,4,m+1) + bdem(:,4,2,4,m+1);
        fde(:,5,m+1) = fdem(:,1,3,m+1) + fdem(:,2,4,m+1) + fdem(:,3,1,m+1) + fdem(:,4,2,m+1);       
% end of random variables loop
  end

% compute G-matrices and rhs-multipliers
  G = stoch_gmatrices_new(indset1,indset2);
  rhs_ind = zeros(1,norv+1); %% this is ok IF rhs is deterministic and, as always assumed, (0,0,...,0) is not in indset2;
                             %% IF rhs is stochastic then rhs_ind should be computed using
                             %% [rhs_ind,beta] = stoch_rhs_multipliers(indset2,P2,norv);

% assemble interior residuals from rhs-contributions
  for m=0:noarv
      ind_m = rhs_ind(1,m+1);
      if ind_m>0
         intres(:,(5*(ind_m-1)+1):(5*ind_m)) = intres(:,(5*(ind_m-1)+1):(5*ind_m)) + fde(:,:,m+1)*beta(1,m+1); % LR modification
      end
  end
  
% assemble interior residuals from div(a*grad)-contributions
%
%   compute local matrices and multiply them by the solution vector
  for m = 0:noarv
      [nzi,nzj,g] = find(G{m+1});
      for t=1:length(nzi)
          i=nzi(t); j=nzj(t);
          for ii = 1:5
              for jj = 1:4
                intres(:,5*(i-1)+ii) = ...
                intres(:,5*(i-1)+ii) + (g(t)*bde(:,ii,jj,m+1)).*sl_v(:,4*(j-1)+jj);
              end
          end
      end
  end
  
  return

