function [intres] = stoch_intres_q2_yq(xy,mv,q2sol,indset1,P1,indset2,norv,noarv,KL_DATA)
%STOCH_INTRES_Q2_YQ  computes YQ elementwise interior residuals for stochastic Q2 solution
%   [intres] = stoch_intres_q2_yq(xy,mv,q2sol,indset1,P1,indset2,norv,noarv,KL_DATA);
%   input
%          xy         vertex coordinate vector
%          mv         element mapping matrix
%          q2sol      stochastic Q1 solution vector
%          indset1    first index set (for the stochastic solution)
%          P1         length of the first index set
%          indset2    second index set (for the error estimator)
%          norv       number of random variables
%          noarv      the number of active random variables
%          KL_DATA    data related to KL-expansion
%   output
%          intres     interior residuals
%
%   calls functions gausspoints_oned, gausspoints_twod, deriv, qderiv, qqderiv
%   calls functions stoch_gauss_source, stoch_gauss_gradcoeff, stoch_rhs_multipliers
%   SIFISS function: DJS 2 October 2017 (modifications of T. Gergelits)
% Copyright (c) 2013 A. Bespalov, C.E. Powell, D.J. Silvester



%% taken from diffpost_q2_with_q4
reduction_level=3;
error_node=[10;11;12;13;14;15;16;17;18;20;22;24];
N_error_node=length(error_node); nn=N_error_node;

  x=xy(:,1); y=xy(:,2); nvtx=length(x);
  mel=length(mv(:,1));
  [P2,norv] = size(indset2);

% construct the integration rule
  ngpt=7;
  [oneg,onew] = gausspoints_oned(ngpt);
  [s,t,wt] = gausspoints_twod(oneg,onew);
  nngpt=ngpt^2; ng=ngpt;

% inner loop over elements
  for ivtx = 1:4
      xl_v(:,ivtx) = x(mv(:,ivtx));
      yl_v(:,ivtx) = y(mv(:,ivtx));
      end
   for ivtx=1:9
       for k=1:P1
          sl_v(:,ivtx+9*(k-1)) = q2sol(nvtx*(k-1)+mv(:,ivtx));
      end
  end

% initialisation
  intres = zeros(mel,nn*P2);
  fde = zeros(mel,nn,noarv+1);
  bde = zeros(mel,nn,9,noarv+1);
%


% loop over Gauss points
   for igpt = 1:nngpt
   sigpt=s(igpt);
   tigpt=t(igpt);
   wght=wt(igpt);
% evaluate derivatives etc
   [jac_v,invjac_v,phi_v,dphidx_v,dphidy_v] = deriv(sigpt,tigpt,xl_v,yl_v);
   rhs = stoch_gauss_source(sigpt,tigpt,xl_v,yl_v,noarv);
   [qua,dquadx,dquady] = qqderiv(sigpt,tigpt,xl_v,yl_v);
   coeff = stoch_gauss_coeff(sigpt,tigpt,xl_v,yl_v,noarv,KL_DATA);
   [dcoeffdx,dcoeffdy] = stoch_gauss_gradcoeff(sigpt,tigpt,xl_v,yl_v,noarv,KL_DATA);
   [psi,dpsidx,dpsidy] = qderiv(sigpt,tigpt,xl_v,yl_v);
   [dpsi2dx2,dpsi2dy2] = qderiv_2(sigpt,tigpt,xl_v,yl_v);
   deltapsi = dpsi2dx2+dpsi2dy2;
      for m = 0:noarv
         for j = 1:N_error_node
         jj=error_node(j);
% compute rhs-contributions
         fde(:,j,m+1) = fde(:,j,m+1) + wght*rhs(:,m+1) .* qua(:,jj) .* jac_v(:);
% compute div(a*grad)-contributions
            for i = 1:9  % element basis functions
            bde(:,j,i,m+1) = bde(:,j,i,m+1)+ wght*dcoeffdx(:,m+1).*dpsidx(:,i).*qua(:,jj);
            bde(:,j,i,m+1) = bde(:,j,i,m+1)+ wght*dcoeffdy(:,m+1).*dpsidy(:,i).*qua(:,jj);
            bde(:,j,i,m+1) = bde(:,j,i,m+1)+ wght*coeff(:,m+1).*deltapsi(:,i).*qua(:,jj).*invjac_v(:);
            end
         end
      end
% end of Gauss point loop
   end

% compute G-matrices and rhs-multipliers
  G = stoch_gmatrices_new(indset1,indset2);
  rhs_ind = zeros(1,norv+1); %% this is ok IF rhs is deterministic and, as always assumed, (0,0,...,0) is not in indset2;
                             %% IF rhs is stochastic then rhs_ind should be computed using
                             %% [rhs_ind,beta] = stoch_rhs_multipliers(indset2,P2,norv);

% assemble interior residuals from rhs-contributions
  for m=0:noarv
      ind_m = rhs_ind(1,m+1);
      if ind_m>0
         intres(:,(nn*(ind_m-1)+1):(nn*ind_m)) = intres(:,(nn*(ind_m-1)+1):(nn*ind_m)) + fde(:,:,m+1)*beta(1,m+1); %DJS
      end
  end

% assemble interior residuals from div(a*grad)-contributions
%

%   compute local matrices and multiply them by the solution vector
for m = 0:noarv
    [nzi,nzj,g] = find(G{m+1});
    for t=1:length(nzi)
        i=nzi(t); j=nzj(t);
        for ii = 1:nn
            for jj = 1:9
                intres(:,nn*(i-1)+ii) = ...
                intres(:,nn*(i-1)+ii) + (g(t)*bde(:,ii,jj,m+1)).*sl_v(:,9*(j-1)+jj);
            end
        end
    end
end
  
return

